select partman.create_parent( 'public.deliveries_finished', 'created_at', 'native', 'daily', p_premake := 20) ;
)))))__SQLFMT_OUTPUT__(((((
select
    partman.create_parent(
        'public.deliveries_finished', 'created_at', 'native', 'daily', p_premake := 20
    )
;
