explain select top 25
foooooooo, barrrrrrrrr, bazzzzzzzzzzzzzzzz, quxxxxxxxxxxx, foooooooooooooooooo, barrrrrrrrr
from something
)))))__SQLFMT_OUTPUT__(((((
explain
select top 25
    foooooooo,
    barrrrrrrrr,
    bazzzzzzzzzzzzzzzz,
    quxxxxxxxxxxx,
    foooooooooooooooooo,
    barrrrrrrrr
from something
