-- https://github.com/tconbeer/sqlfmt/issues/637
SELECT e.col:['attr_ID']::string as id
FROM expl e
)))))__SQLFMT_OUTPUT__(((((
-- https://github.com/tconbeer/sqlfmt/issues/637
select e.col:['attr_ID']::string as id from expl e
