select 0;
-- fmt: off
select 1;
-- fmt: off
select 2;
-- unsupported ddl
CREATE PUBLICATION mypublication FOR TABLE users, departments;
-- fmt: on
select 3;
-- fmt: on
select 4;
-- fmt: on
-- fmt: on
-- fmt: on
-- fmt: on
