something
as
(
    select
        1
)
+
1
+
1
+
1,
something_else,
count(
    *
)
over
(
),
"a"
||
"b"
|| a
)))))__SQLFMT_OUTPUT__(((((
something
as (
    select
        1
)
+ 1
+ 1
+ 1,
something_else,
count(
    *
)
over (
),
"a"
|| "b"
|| a
