select 
several_fields, several_fields, several_fields, several_fields, several_fields
from 
    my_model;
    select one_liner;
select several_fields, several_fields, 
several_fields, several_fields, several_fields
from my_model
where 
    something and something_else
    ;
)))))__SQLFMT_OUTPUT__(((((
select several_fields, several_fields, several_fields, several_fields, several_fields
from my_model
;
select one_liner
;
select several_fields, several_fields, several_fields, several_fields, several_fields
from my_model
where something and something_else
;
